#!/usr/bin/env ruby -w

# package a raw code block, assembled to run from 23766, as a .tap file containing a single Program: block that runs that code

# convert val to a two byte string of that number as a little-endian word
def le16(value)
	(value % 256).chr + (value / 256).chr
end

# return the byte sequence that the Spectrum would save for the given flag byte and data block
def zxblock(flag_byte, data)
	checksum = flag_byte
	data.each_byte do |b|
		checksum ^= b
	end
	return flag_byte.chr + data + checksum.chr
end

# return the byte sequence to be included in a tap file for the given flag byte and data block
def tapblock(flag_byte, data)
	zxdata = zxblock(flag_byte, data)
	return le16(zxdata.length) + zxdata
end

if ARGV.length != 3:
	puts "Usage: bin2rem.rb input.bin zxfilename output.tap"
	exit
end

infile, filename, outfile = ARGV

f = File.open(infile, "rb")
data = f.read
f.close

# line 1, length 4096 (arbitrary), RANDOMIZE USR [displayed 0, actually 23766], data
basic = "\x00\x01" + le16(4096) + "\xf9\xc00\x0e\x8f\x39\xac" + data

# Program: [filename padded to 10]
filename = (filename + " "*10)[0..9]
header = "\x00" + filename + le16(basic.length) + le16(1) + le16(basic.length)

File.open(outfile, "wb") do |f|
	f.print tapblock(0x00, header)
	f.print tapblock(0xff, basic)
end

puts "Written #{outfile} - Basic block is #{basic.length} bytes"
