	org 0x7800

align	macro n
	org ($ + n - 1) / n * n
	endm

	ld sp,0x7800
	jp launch

	; == begin stuff that can safely be in low memory ==

	include "stc_player.asm"
music_data
	incbin "trixies_magic_unicorn.stc"
	; == end low-memory stuff ==

launch
	ld hl,music_data
	call stc_init

rom_font equ 0x3d00
font equ 0x6000

; == generate half-bold font ==
	ld hl,rom_font
	ld de,font

	ld b,96
boldfont_lp
	rept 4
		ld a,(hl)
		add a,a
		or (hl)
		ld (de),a
		inc hl
		inc de
		dec bc
	endm
	rept 4
		ld a,(hl)
		ld (de),a
		inc hl
		inc de
	endm
	djnz boldfont_lp
; == end generate half-bold font ==

; == Generate sine table ==
sine	equ 0xbf00
CALC_DELETE		equ 0x02
CALC_MULTIPLY	equ 0x04
CALC_ADDITION	equ 0x0f
CALC_SIN		equ 0x1f
CALC_INT		equ 0x27
CALC_DUPLICATE	equ 0x31
CALC_CONST		equ 0x34
CALC_STK_ZERO	equ 0xa0
CALC_ST_MEM_0	equ 0xc0
CALC_END_CALC	equ	0x38

	rst 0x28
	db CALC_STK_ZERO
	db CALC_END_CALC

	ld hl,sine
sinelp
	push hl

	; progress bar
	srl l
	srl l
	srl l
	ld h,0x58
	ld (hl),h

	rst 0x28
; the accurate version
;	db CALC_CONST, 0xeb, 0x49, 0x0f, 0xda, 0xd8 ;const pi/128
; the less accurate version
;	db CALC_CONST, 0x6b, 0x49, 0x10 ;const pi/128
; the inaccurate but good enough version
	db CALC_CONST, 0x2b, 0x49 ;const pi/128
	db CALC_ADDITION
	db CALC_DUPLICATE
	db CALC_SIN
	db CALC_CONST, 0x37, 0x7e		; const 127
	db CALC_MULTIPLY
	db CALC_INT
	db CALC_ST_MEM_0
	db CALC_DELETE
	db CALC_END_CALC
	ld a,(iy+0x5a)			; third byte of calculator memory 0
	pop hl
	ld (hl),a
	inc l
	jr nz,sinelp
; == END generate sine table ==

sine8	equ 0xbc00
; generate scaled down to +/- 8 sine table
	ld de,sine8
sine8_lp
	ld a,(hl)
	sra a
	sra a
	sra a
	sra a
	ld (de),a
	inc l
	inc e
	jr nz,sine8_lp

sumsine	equ 0xbb00
; generate sum-of-sines table
	ld hl,sine8
	ld bc,sine8
	ld de,sumsine
sumsine_lp
	ld a,(bc)
	add a,(hl)
	ld (de),a
	inc c
	ld a,l
	add a,32
	ld l,a
	inc e
	jr nz,sumsine_lp

	; clear both screens
	call swap_screen
	call clear_screen
	call swap_screen
	call clear_screen

; set up interrupt table
	ld hl,0xbd00
	ld de,0xbd01
	ld (hl),0xbe
	ld bc,0x0100
	ldir
	ld a,0xc3 ; opcode for JP
	ld (0xbebe),a
	ld hl,interrupt
	ld (0xbebf),hl
	di
	ld a,0xbd
	ld i,a
	im 2
	ei

; == plot macro ==
; enter with l=y coord, c=x coord
; preserves bc
	macro plot_lc
	ld h,high ypositions
	ld e,(hl)
	inc h
	ld d,(hl)
	inc h
	ld l,c
	ld a,(hl)
	inc h
	or e
	ld e,a
	ld a,(hl)
	ex de,hl
	or (hl)
	ld (hl),a

	ld (ix),l ; save in plotlist
	inc ix
	ld (ix),h
	inc ix
	endm

; as above, but y coord in b (bc = y,x)
	macro plot
	ld l,b
	plot_lc
	endm

; == end plot ==

loop
	halt
;	ld a,2
;	out (254),a

	call swap_screen

	call unplot

	ld ix,(plotlist+1)

wavepos
	ld iy,sine8 ; sumsine
	ld a,iyl
	add a,8
	ld iyl,a
	ld (wavepos+2),iy

	; plot a rotating thing
rota_angle
	ld a,0
	inc a
	inc a
	ld (rota_angle+1),a
	ld h,high sine
	ld l,a
	ld a,(hl)	; a = sin(rota_angle)
	sra a
	ld e,a ; e = 0.5sin(rota_angle)

	ld a,l
	add a,64
	ld l,a
	ld a,(hl)	; a = cos(rota_angle)
	sra a
	ld d,a ; d = 0.5cos(rota_angle)

	; inner (x) loop: x coord increments by (cos r)/16 each time, y coord increments by (sin r)/32 each time
	; outer (y) loop: x coord increments by -(sin r)/16 each time, y coord increments by (cos r)/32 each time
	; starting position: x = -(cos r)/2 + (sin r)/2 + 0x80, y = -(cos r)/4 - (sin r)/4 + 0x40
; hl' = y coord, de' = ycoord offset for x motion (= sin r)
; hl = x coord, de = xcoord offset for x motion (= cos r)

	; row_dy = amount to increment y for each step in a row = sinr/32
	ld h,e
	ld l,0
	sra h
	rr l
	sra h
	rr l
	sra h
	rr l
	sra h
	rr l
	ld (row_dy+1),hl

	; row_dx = amount to increment x for each step in a row = cosr/16
	ld h,d
	ld l,0
	sra h
	rr l
	sra h
	rr l
	sra h
	rr l
	ld (row_dx+1),hl

	; nextrow_dy = amount to increment y between rows = cosr/32
	sra h
	rr l
	ld (nextrow_dy+1),hl

	; nextrow_dx = amount to increment x between rows = -sinr/16
	ld a,e
	neg
	ld h,a
	ld l,0
	sra h
	rr l
	sra h
	rrl
	sra h
	rr l
	ld (nextrow_dx+1),hl

	; initial X = sinr/2 - cosr/2 + 0x80
	ld a,d
	neg ; a = -cosr/2
	add a,e ; a = sinr/2-cosr/2
	add a,0x80
	ld h,a
	ld l,0 ; now hl = initial X as 8.8

	; initial Y = -cosr/4 - sinr/4 + 0x40 = (-cosr/2 - sinr/2)/2 + 0x40
	ld a,d
	neg ; a = -cosr/2
	sub e ; a = -cosr/2-sinr/2
	sra a
	add a,0x40
	exx
	ld h,a
	ld l,0
	exx ; now hl' = initial Y as 8.8

rowcount
	ld c,14
	; == rotasqr outer loop ==
rotasqr_outer_lp
	; save initial hl, hl'
	push hl
	exx
	push hl
row_dy ld de,0
	exx

	push bc ; save c
	ld b,14
	; == rotasqr inner loop ==
rotasqr_row_lp

	; gather x,y in c,l
	exx
	add hl,de
	ld a,h
	exx
	ld c,h
	push hl
	add a,(iy)
	inc iyl
	ld l,a
	plot_lc

	pop hl
row_dx ld de,0
	add hl,de

	djnz rotasqr_row_lp
	; == end rotasqr inner loop ==

	inc iyl
	inc iyl

	pop bc ; restore c

	; restore initial hl, hl'
	exx
	pop hl
nextrow_dy
	ld de,0
	add hl,de
	exx
	pop hl
nextrow_dx
	ld de,0
	add hl,de
	dec c
	ld a,c
	or a
	jp nz,rotasqr_outer_lp
	; == end rotasqr outer loop ==

	; end marker for plotlist - high byte is 0
	xor a
	ld (ix+1),a

	ld a,5
	out (254),a

	jp loop

printchar
printchar_screenpos
	ld de,0xd020
printchar_textpos
	ld hl,text
	ld a,(hl)
	or a
	ret z
	push hl
	ld l,a
	ld h,0
	add hl,hl
	add hl,hl
	add hl,hl
	ld bc,font-0x100
	add hl,bc
	
	push de
	rept 7
	ld a,(hl)
	ld (de),a
	inc l
	inc d
	endm
	ld a,(hl)
	ld (de),a
	pop de

	pop hl
	; increment text pos and screen pos every second frame
printchar_inc
	ld a,0
	xor 1
	ld (printchar_inc+1),a
	ret z
	inc hl
	ld (printchar_textpos+1),hl
	inc e
	ld (printchar_screenpos+1),de
	ret

unplot
	ld (unplot_spback+1),sp
	ld sp,(plotlist+1)
	xor a
unplot_lp
	pop hl
	ld (hl),a
	cp h
	jp nz,unplot_lp
unplot_spback
	ld sp,0
	ret

swap_screen
curr_screen
	ld a,0x17 ; page 7 paged in, screen 5 active
	xor 0x0a ; swap screen (bit 3) and bit 1 of page
	ld (curr_screen+1),a
	ld bc,0x7ffd
	out (c),a
plotlist
	ld hl,0xb400 ; alternate between 0xb4 and 0xb8; xor %00001100
	ld a,h
	xor %00001100
	ld h,a
	ld (plotlist+1),hl
	ret

clear_screen
	ld hl,0xc000
	ld de,0xc001
	ld (hl),l
	ld bc,0x1800
	ldir
	ld (hl),%00101001 ; blue on cyan
	ld bc,0x2ff
	ldir
	ret

interrupt
	push af
	push bc
	push de
	push hl
	push ix
	call stc_play
	call texthandler
	pop ix
	pop hl
	pop de
	pop bc
	pop af
	ei
	ret

texthandler
th_pos
	ld hl,1023
	dec hl
	ld a,h
	and 0x03
	ld h,a
	ld (th_pos+1),hl
	; if hl > 128, call printchar
	jp nz,printchar
	ld a,l
	cp 128
	jp nc,printchar

	; 14 13 12 11 10 9 8 7 6 5 4 3 2 1 1 2 3 4 5 6 7 8 9 10 11 12 13 14
	cp 28*2
	jr nc,no_switch_fx
	push af
	srl a
	jr c,switchfx_skip_odd
	sub 14
	jr c,switchfx_neg
	jr nz,no_switch_table

whichtable
	ld a,1
	xor 1
	ld (whichtable+1),a
	ld hl,sumsine
	jr z,swich_table_done
	ld hl,sine8
swich_table_done
	ld (wavepos+2),hl

no_switch_table
	inc a
	jr switchfx_done
switchfx_neg
	neg
switchfx_done
	ld (rowcount+1),a
switchfx_skip_odd
	pop af

no_switch_fx
	or a
	jp nz,skip_newpage
	ld hl,(printchar_textpos+1)
	inc hl
	ld a,(hl)
	or a
	jr nz,text_nowrap
	ld hl,text
text_nowrap
	ld (printchar_textpos+1),hl
	ld hl,0xd020
	ld (printchar_screenpos+1),hl
	xor a
skip_newpage
; clear the row number in 'a'/2
	srl a
	neg
	add a,192
	ld l,a
	ld h,high ypositions
	ld e,(hl)
	inc h
	ld d,(hl)
	xor a
	ld b,32
clear_row
	ld (de),a
	inc de
	djnz clear_row
	ret

text
	   ;01234567890123456789012345678901
	db "      -=- Hooy-Program -=-      "
	db "                                "
	db "            presents            "
	db "                                "
	db "    h a i r y   p l o t t e r   "
	db "                                "
	db "  a Speccy 4k for Revision 2012 "
	db 0
	db "g'day folks. Since we went to   "
	db "all this trouble hooking up my  "
	db "Spectrum to the recording       "
	db "equipment, I figured I ought to "
	db "make it worthwhile by actually  "
	db "doing a Speccy release. So, hope"
	db "you enjoy this bit of partycode!"
	db 0
	db "Greetificationings to... ArtWaY,"
	db "Adinpsz, Approximate, CRTC, 3LN,"
	db "Brainstorm, Accession, #ukscene,"
	db "Unstable Label, RBBS, Fairlight,"
	db "Farbrausch, Concern Chaos, TRSI,"
	db "Loonies, Benediction, & everyone"
	db "who didn't fit in this space :-P"
	db 0
	db "Credits:                        "
	db "Code......................gasman"
	db "Music................also gasman"
	db "                                "
	db "(c) 2012 Hooy-Program           "
	db "               for Revision 2012"
	db 0,0

;table of screen addresses.
; First 256 (actually 192) bytes are the low byte, next 256 are the corresponding high byte
; NB compresses to ~80 bytes. Can probably generate it in less than that, if we're short on bytes...
	align 0x100
ypositions
	rept 256, n
	db (n & 0x38)<<2
	endm
	rept 192, n
	db 0xc0|(n & 0x07)|(n & 0xc0)>>3
	endm
	ds 64
; divide-by-8 table
	rept 256, n
	db n/8
	endm
xpositions
	rept 32
	db 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01
	endm
